Origin

Struct Origin 

pub struct Origin<'src> {
    pub span: Span,
    pub raw: &'src str,
}
Represents the origin of an expression, including its source span and unparsed form.

Fields

span: Span

The expression’s source span.

raw: &'src str

The expression’s unparsed form, as it appears in the source.

This is recorded exactly as it appears in the source, except that leading and trailing whitespace is stripped. This is stripped because it’s (1) non-semantic, and (2) can cause all kinds of issues when attempting to map expressions back to YAML source features.

Implementations

Source

impl<'a> Origin<'a>

pub fn new(span: impl Into<Span>, raw: &'a str) -> Self

Create a new origin from the given span and raw form.